import numpy as np
import scipy.stats
import matplotlib.pyplot as plt
import tabular as tb

import tools.qFunctions as q
import core.init
import behav


def doPairwise(rp, allData1, allData2):

    agg1 = allData1[(allData1['cond'] != 0) & (allData1['accuracy'] != 'No response')].aggregate(
        On = ['subjID','cond'],
        AggFuncDict = {'RT': np.mean})[['subjID','cond','RT']]
    #import pdb; pdb.set_trace()
    conds = [1,2,3,4]#[5,6,7,8]
    #agg1.aggregate(On = ['cond'])['cond'].extract()
    
    plotData1 = []
    for subjID in rp.subjID_list:
        thisAgg = agg1[agg1['subjID']==subjID]
        combs = q.combinations(conds,2)
        for c in combs:
            
            diff = thisAgg[thisAgg['cond']==c[0]]['RT'] - thisAgg[thisAgg['cond']==c[1]]['RT']
            plotData1.append((subjID, c[0], c[1], diff[0]))
    
    plotData1 = tb.tabarray(records = plotData1, names = ['subjID', 'c1', 'c2', 'diff'])
    plotData1.sort(order=['subjID','c1','c2'])
    
    agg2 = allData2[allData2['type'] == 'whole -> whole'].aggregate(
        On = ['subjID','ROI','train1', 'train2'],
        AggFuncDict = {'LinearNuSVMC': np.mean})[['subjID','ROI','train1', 'train2','LinearNuSVMC']]
    agg2.sort(order=['ROI'])    
    ROIs = agg2.aggregate(On = ['ROI'])['ROI'].tolist()
    plotData2 = agg2
    
    #import pdb; pdb.set_trace()
    #allData['type'] == 'whole -> whole'
    
    #plotData = np.sum(np.array(agg),0)/numYerr
    N = len(rp.subjID_list)
    fig = plt.figure()
    fig.canvas.set_window_title('Results')
    for i, ROI in enumerate(ROIs):
        pd1 = plotData1['diff']
        pd2 = plotData2[plotData2['ROI']==ROI]['LinearNuSVMC']
        r, p = scipy.stats.pearsonr(pd1,pd2)
        fig.add_subplot(3,3,i+1, title = ROI[1]+', Pearson r = %.2f, ' %r + 'p = %.3f')
        pdr1 = np.reshape(pd1,(N,-1))
        pdr2 = np.reshape(pd2,(N,-1))
        plt.plot(pdr1, pdr2, 'o')
    plt.show()
    
    
def run(
    rp,
    data1 = None,
    data2 = None
    ):
    
    """
    Calculates and plots a correlation between two data sets (per subject).
    Currently implemented such that data1 is behavioral data and data2 is SVM.
    
    **Parameters**
        rp: Namespace (required)
            Run parameters that are parsed from the command line
        data1: str or *None*
        data2: str or tabarray
    
    """
    
    # behavioral data processing
    if type(data1) == str: # file name specified
        allData1 = tb.tabarray(SVfile = core.init.paths[rp.moduleName] + data1)
    else:
        allData1 = behav.getBehavData(rp, dataFile = data1)
    
    # calculate RT mean for each subject each context
    agg1 = allData1[(allData1['cond'] != 0) & (allData1['accuracy'] != 'No response')].aggregate(
        On = ['subjID','context'],
        AggFuncDict = {'RT': np.mean})[['subjID','context','RT']]
    
    # calculate RT differences Parts - Whole
    plotData1 = []
    for subjID in rp.subjID_list:
        thisAgg = agg1[agg1['subjID']==subjID]
        diff = thisAgg[thisAgg['context']=='None']['RT'] - thisAgg[thisAgg['context']=='Good']['RT']
        plotData1.append((subjID, diff[0]))
    
    plotData1 = tb.tabarray(records = plotData1, names = ['subjID', 'diff'])
    plotData1.sort(order=['subjID'])
    
    
    # SVM data processing
    if type(data2) == str: # file name specified
        allData2 = tb.tabarray(SVfile = core.init.paths[rp.moduleName] + data2)
    else:
        allData2 = data2
    
    # calculate mean classification performance for each subject each ROI each context    
    agg2 = allData2.aggregate(
        On = ['subjID','ROI','type'],
        AggFuncDict = {'LinearNuSVMC': np.mean})[['subjID','ROI','type','LinearNuSVMC']]
    
    #ROIs = agg2.aggregate(On = ['ROI'])['ROI'].tolist()    
    
    # calculate percent correct differences Parts - Whole
    plotData2 = []
    for subjID in rp.subjID_list:
        thisAgg = agg2[agg2['subjID']==subjID]
        for ROI in rp.rois:
            thisROIagg = thisAgg[thisAgg['ROI']==ROI[1]]
            diff = thisROIagg[thisROIagg['type']=='parts']['LinearNuSVMC'] -\
                thisROIagg[thisROIagg['type']=='whole']['LinearNuSVMC']
            plotData2.append((ROI[1], subjID, diff[0]))
    plotData2 = tb.tabarray(records = plotData2, names = ['ROI','subjID','diff'])
    plotData2.sort(order=['ROI'])    
    
    # plot a correlation between the two    
    fig = plt.figure()
    fig.canvas.set_window_title('Results')
    
    for i, ROI in enumerate(rp.rois):
        pd1 = plotData1['diff']
        pd2 = plotData2[plotData2['ROI']==ROI[1]]['diff']
        r, p = scipy.stats.pearsonr(pd1,pd2)
        fig.add_subplot(3,3,i+1, title = ROI[1] + ', Pearson r = %.2f, ' %r + 'p = %.2f' %p)
        plt.ylim((-.3,.3))
        pdr1 = np.reshape(pd1,(len(rp.subjID_list),-1))
        pdr2 = np.reshape(pd2,(len(rp.subjID_list),-1))
        plt.plot(pdr1, pdr2, 'o', c='k')
    fig.set_size_inches((15,15), forward = True)
    
    if rp.savePlot != None: plt.savefig(rp.savePlot, bbox_inches = 'tight')
    if not rp.textOnly: plt.show()