import core.init
import tools.qFunctions as q
import svm
import core.exp
import dataAnalysis as da

import numpy as np
import matplotlib.pyplot as plt
import tabular as tb
import random
import os


def extractPSC(
    nim,
    window = 5, # in TRs, not seconds (window*TR = time)
    shiftTp = 2
    ):
    """
    Extracts percent signal change within a given window
    """
    
    meanNim = np.mean(nim.samples,axis=1)
    timeCourse = {}
    for no,lb in enumerate(nim.labels):
        if not lb in timeCourse.keys(): timeCourse[lb] = []
        thisWin = meanNim[no-shiftTp:no-shiftTp+window]
        chunks = nim.C[no-shiftTp:no-shiftTp+window] # to make sure we don't go over each run
        # check if we don't go over each run boundary
        if len(thisWin) == window and np.mean(chunks) == chunks[0]:
            timeCourse[lb].append(thisWin)
        # else just ignore those timepoints
    
    for key, value in timeCourse.items(): timeCourse[key] = np.array(value)
        
    baseline = np.mean(timeCourse[0], axis = 0) # baseline is condition 0
    psc = np.zeros((window,len(timeCourse.keys())-1)) # time by condition
    no = 0
    for key, value in timeCourse.items():
        if key != 0: # no fixation
            psc[:,no] = (np.mean(value,axis=0) - baseline)/baseline*100
            no += 1
            
    return psc


def run(
    rp,
    allConds = q.OrderedDict([
        ('parts', [1,2,3,4]),
        ('whole', [5,6,7,8])
    ]),
    window = 5,
    filter = None,
    plotTp = None
    ):    
    
    if rp.i != None: # simply plot from the existing analysis file
        plotData = tb.tabarray(SVfile = rp.i)
    else:
        plotNames = ['subjID','ROI','ROIfiles','numVoxels','context','time','PSC']
        rp.rois_p = q.genFileList(rp.rois)
        
        plotData = []
        for subjIDno, subjID in enumerate(rp.subjID_list):
            print ''
            print subjID
        
            # read in data files
            allDataFiles = q.listDir(core.init.paths['dataBehavDir'] %subjID,\
                pattern = '.*' + rp.runType + '\.csv', fullPath = True)
            
            # put them all in a tabarray
            first = True
            for thisDataFile in allDataFiles:
                print 'Loading %s' %thisDataFile        
                # load the appropriate data file to get labels (conditions)
                if first:
                    tabData = tb.tabarray(SVfile = thisDataFile)
                    first = False
                else: tabData = tabData.rowstack(tb.tabarray(SVfile = thisDataFile))
        
            
            for ROI_list in rp.rois:
                thisNim = svm.makeNim(subjID, rp.runType, tabData, ROI_list[2],
                    filter = filter, shiftTp = core.exp.shiftTp)
                psc = extractPSC(thisNim, window = window, shiftTp = core.exp.shiftTp)
                
                # collect all ROI names
                allROIs = []
                for thisROI in ROI_list[2]:
                    allROIs.extend(q.listDir(core.init.paths['recDir'] %subjID, \
                        pattern=thisROI+'\.nii', cropExt = True, fullPath = False))
                            
                rec = [
                    subjID,
                    ROI_list[1], # ROI names combined together or specified by the user
                    ';'.join(allROIs), # ROI file names
                    thisNim.samples.shape[1]
                    ]
                
                for context, conds in allConds.items(): # go through each condition
                    conds = [c-1 for c in conds]
                    for j, thisPSC in enumerate(psc): # each timepoint within that condition
                        plotData.append(tuple(rec + [context, j*2, np.mean(thisPSC[conds])]))
        
        plotData = tb.tabarray(array = plotData, names = plotNames)
        if not rp.noOutput: plotData.saveSV(rp.o)
        
    
    if plotTp != None:
        da.plotTabular(
            plotData[plotData['time'] == plotTp],
            xAxis = ['ROI', [item[1] for item in rp.rois]],
            categories = ['context', allConds.keys()],
            values = 'PSC',
            yerrName = 'subjID',
            plotType = 'bar',
            textOnly = rp.textOnly,
            savePlot = rp.savePlot,
            userOpt = {
                'colors': 'gray1',
                'hatch': ['','','/','x'],
                'subplot': {
                    'title': 'percent signal change',
                    'xlabel': '',
                    'ylim': (0,.4)
                    }
                }
            )
    else:
        for r, thisROI in enumerate(rp.rois):
            if r == 0: fig = None
            fig = da.plotTabular(
                plotData[plotData['ROI'] == thisROI[1]],
                fig = fig,
                subplotno = 231+r,
                xAxis = ['time',range(0,2*window,2)],#range(0,thisNim.getDt()*window,thisNim.getDt())],
                categories = ['context', allConds.keys()],
                values = 'PSC',
                yerrName = 'subjID',
                plotType = 'errorbar',
                textOnly = True,#rp.textOnly,
                savePlot = None,
                userOpt = {
                    'colors': 'gray1',
                    'subplot': {
                        'title': thisROI[1],
                        'xlabel': 'time after stimulus onset, s',
                        'ylabel': 'percent signal change',
                        'ylim': (-.1,.5)
                        }
                    }
                )
        fig.subplots_adjust(left = .1, right = .95, bottom = .1, top = .95)
        fig.set_size_inches((8,7), forward = True)
        if rp.savePlot != None: plt.savefig(rp.savePlot, bbox_inches = 'tight')
        plt.show()
                    # 