###############################################################################

"""
Presents horizontal and vertical wedges for medirian mapping
Task: no task
"""


# Get default experimental flow variables and functions
import sys, os
import core
import core.exp as exp


def runExp(rp):

    ## Import modules #########################################################

    # Include PsychoPy and data manipulation modules
    from psychopy import visual
    import numpy as np
    import tabular as tb

    # Include other stuff
    import random
    import tools.qFunctions as q


    ## Variables ##############################################################    
    paraType = 'blocked'
    # fMRI parameters
    exp.numTrialsInBlock = 20
    
    #          CondNo Condition
    exp.paraTable = {0: ('Fixation',  ),
                     1: ('Objects',   ),
                     2: ('Scrambled', )}

    exp.validResponses = {exp.validResponses.keys()[0]:
                          exp.validResponses.values()[0]}
    #         CondNo Pos
    posTable = {1: 'Top Left',
                2: 'Top Right',
                3: 'Bottom Left',
                4: 'Bottom Right',
                5: 'Top Left',
                6: 'Top Right',
                7: 'Bottom Left',
                8: 'Bottom Right'}

    # Stimuli parameters
    stimSize = 5
    stimDist = 5 # from the fixation in x or y dir
    # directory where images are coming from
    imageDir = os.path.dirname(__file__) + '/images/'

    stimSizeHalf = stimSize/2

    # Define stimuli
    exp.createBasicStim(rp.debug)
    
    image = visual.PatchStim(
        exp.win,
        size = stimSize,
        sf = 1.0/stimSize # for some reason this has to be specified
    )
    
    # All stimuli
    s = {0: [exp.fixation],
         1: [image]}
         
    # Positions of all stimuli
    p = {0: (        0,        0), # Fixation position
         1: (-stimDist, stimDist),
         2: ( stimDist, stimDist),
         3: (-stimDist,-stimDist),
         4: ( stimDist,-stimDist)}
        
    events = {1: (300, [1, 2, 3, 4, 0]),
              0: (500, [0            ])}
    trial = [events[1], events[0]]
    exp.trialDur = sum(event[0] for event in events.values())/1000.
    

    ## Functions ##############################################################    
    def assignFun(lineNo, thisLine, ind, pos):
        key = thisLine['posNo']
        ind2 = str(((key-1)%4 == ind) + 1)
        return [s[1],
            ["setPos(" + str(p[pos])+")",
             "setTex('" + thisLine['image'+ind2] + "')"]]
             

    ## Main flow ##############################################################
    
    # Generate a list of image pairs and display positions
    expPlan = exp.prepareExpPlan(rp, paraType,
        otherDescr = [('posNo', '<i4'), ('pos', '|O4'),
                 ('image1', '|O4'), ('image2', '|O4')])
    
    # Define where images for each condition should come from
    allImagesList = {
        1: q.listDir(imageDir + 'objects', '.jpg'),
        2: q.listDir(imageDir + 'scrambled','.jpg')}
    
    expPlan['posNo'] = 0
    expPlan['pos'] = posTable.get(0)
    expPlan['image1'] = ''
    expPlan['image2'] = ''
    # Generate a list of image pairs and display positions
    for thisBlockNo in expPlan.aggregate(On = 'blockNo')['blockNo']:
        thisBlock = expPlan[expPlan['blockNo'] == thisBlockNo]
        if thisBlock[0]['cond'] != 0:
            thisList = allImagesList[thisBlock[0]['cond']]
            random.shuffle(thisList)
            thisBlock['image1'] = thisList
            thisBlock['image2'] = thisList
            ind = range(len(thisList))
            numTask = random.randint(2, 5)
            replaceList = np.array(random.sample(ind, numTask))
            while np.min(replaceList[1:] - replaceList[:-1]) <= 2:
                replaceList = np.array(random.sample(ind, numTask))
            for i in replaceList:
                randKey = random.choice(posTable.keys())
                thisBlock[i]['posNo'] = randKey
                thisBlock[i]['pos'] = posTable[randKey]
                while thisBlock[i]['image1'] == thisBlock[i]['image2']:
                   thisBlock[i]['image2'] = random.choice(thisList)
            expPlan[expPlan['blockNo'] == thisBlockNo] = thisBlock
            
    expPlan['corrResp'][(expPlan['cond'] != 0) & \
        (expPlan['image1'] != expPlan['image2'])] = exp.validResponses.get(1)
    # Create each trial
    expPlan = exp.makeTrial(expPlan, trial, s, p, assignFun = assignFun)
    # Change full path to images to just a file name
    expPlan['image1'] = q.getFileName(expPlan[['image1']].extract())
    expPlan['image2'] = q.getFileName(expPlan[['image2']].extract())
        
    # Make intro screen and start running
    text = "task: press with your index finger when you see one of the four images being different"
    vertices = [[-stimSizeHalf, -stimSizeHalf],
                [-stimSizeHalf,  stimSizeHalf],
                [ stimSizeHalf,  stimSizeHalf],
                [ stimSizeHalf, -stimSizeHalf]]
    exp.showIntro(text = text, args = (vertices, p, stimSize))
    expPlan = exp.run(expPlan, rp.noOutput)    
    plot(rp)
    exp.dispRunStats(expPlan)

   
    
def plot(rp, plotData = None):
    """
    Plots accuracy and response time
    
    **Parameters**
    
        rp: Namespace (required)
            Run parameters that are parsed from the command line
        plotData: tabarray
            Data that is to be plotted
        
    """
    import analysis.dataAnalysis
    import analysis.behav
    
    if plotData == None: plotData = analysis.behav.getBehavData(rp)
    
    analysis.dataAnalysis.plotTabular(
            plotData,
            filter = "(plotData['cond'] != 0) & (np.logical_not(np.isnan(plotData['corrResp'])))",
            xAxis = 'expName',
            categories = 'descr',
            values = 'accuracy',
            valueFilter = "plotData['accuracy'] == 'Correct'",
            yerrName = 'subjID',
            plotType = 'bar',
            textOnly = rp.textOnly,
            savePlot = rp.savePlot,
            userOpt = {
                'colors': 'gray1',
                'subplot': {
                    'title': 'accuracy',
                    'ylabel': 'proportion correct'
                    }
                }
        )
        
    analysis.dataAnalysis.plotTabular(
            plotData,
            filter = "(plotData['cond'] != 0) & (plotData['accuracy'] == 'Correct') \
                & (np.logical_not(np.isnan(plotData['corrResp'])))",
            xAxis = 'expName',
            categories = 'descr',
            values = 'RT',
            valueFilter = "plotData['accuracy'] == 'Correct'",
            yerrName = 'subjID',
            plotType = 'bar',
            textOnly = rp.textOnly,
            savePlot = rp.savePlot,
            userOpt = {
                'colors': 'gray1',
                'subplot': {
                    'title': 'response time',
                    'ylabel': 'ms'
                    }
                }
        )
         
def run(rp):
    runExp(rp)

    
if __name__ == "__main__":
    run(('confSup', 'test', None, 'loc'), ('False', 'False', 'False'))
