###############################################################################
##                          Meridian mapping script                          ##
###############################################################################

"""
Presents flashing horizontal and vertical wedges with random angular and radial
number of cycles and random color for meridian mapping.
No task.

Author:     Jonas Kubilius
Website:    jonaskubilius.mp
Email:      jonas.kubilius@gmail.com
Created on: 2009-11-09

This script is distributed under the
Creative Commons Attribution-Share Alike 3.0 Unported license
(http://creativecommons.org/licenses/by-sa/3.0)
"""


# Get default experimental flow variables and functions
import sys, os
import core
import core.exp as exp

#exp.paths['paraDirSup'] = os.path.dirname(__file__) + '/' + exp.paths['paraDirSup']

def runExp(rp):
    
    ## Import modules #########################################################

    # Include PsychoPy and data manipulation modules
    from psychopy import visual
    import numpy as np
    import tabular as tb

    # Include other stuff
    import csv
    import random


    ## Variables ##############################################################
    paraType = 'blocked'
    # fMRI parameters
    exp.numTrialsInBlock = 1
       
    #          CondNo  Name
    exp.paraTable = {0: ('Fixation',   ),
                     1: ('Horizontal', ),
                     2: ('Vertical',   )}
    
    exp.validResponses = {}
    
    # Stimuli parameters
    wedgeWidth = 15
    wedgeHeight = 30
    flashPeriod = 1.0/8 # sec; means that there will be 8 changes per sec
    
    # Define stimuli
    exp.createBasicStim(rp.debug)
    wedges = []
    for i in range(4):
        wedges.append(visual.RadialStim(
            exp.win,
            tex             = 'sqrXsqr',
            size            = wedgeHeight,
            visibleWedge    = [90*i, 90*i + wedgeWidth],
            ori             = -wedgeWidth / 2.0
        ))

    # All stimuli
    s = {0: [exp.fixation],
         1: [wedges[1], wedges[3]], # Horizontal
         2: [wedges[0], wedges[2]]} # Vertical

    # Positions of all stimuli
    p = {1: (0, 0)}
    events = {1: (16000, [1, 1])}
    trial = [events[1]]
    
    exp.trialDur = sum(event[0] for event in events.values())/1000.


    ## Functions ##############################################################
    def assignFun(lineNo, thisLine, ind, pos):
        key = thisLine['cond']
        return [s[key], ["setPos("+str(p[pos])+")"]]

    def upsideGauss(mu = 0, sigma = 0.1):
        number = random.gauss(mu, sigma)
        if number < -1: number = -1
        elif number > 1: number = 1
        if number < 0: return -1 - number
        elif number >= 0: return 1 - number
        
    def flashWedge(globClock, trialClock, eventClock,
        expPlan, thisTrial, thisEvent, thisDisplay):
        thres = flashPeriod
        while eventClock.getTime() < thisEvent['dur'] and \
        trialClock.getTime() < thisTrial['dur'] and \
        globClock.getTime() < thisTrial['onset'] + thisTrial['dur']:
            
            exp.lastKey(expPlan = expPlan)
            
            if thisTrial['cond'] != 0:
                if eventClock.getTime() > thres:
                    a = [(upsideGauss(),
                          upsideGauss(),
                          upsideGauss()),
                          random.randint(20, 40),
                          random.randint(4, 18)]
                          
                    for stim in thisDisplay:
                        stim.setRadialPhase((stim.radialPhase + .5) % 1)
                        stim.setRGB(a[0])
                        stim.setAngularCycles(a[1])
                        stim.setRadialCycles(a[2])                            
                        stim.draw()
                    
                    thres += flashPeriod
                    exp.win.flip()

    ## Main flow ##############################################################
    
    # Generate a list of image pairs and display positions
    expPlan = exp.prepareExpPlan(rp, paraType, addBuffer = 4)
    expPlan['corrResp'] = exp.validResponses.get(0) 
       
    # Create each trial
    expPlan = exp.makeTrial(expPlan, trial, s, p, assignFun = assignFun)
    
    # Make intro screen and start running
    exp.showIntro(text = "just stare at the dot")
    expPlan = exp.run(expPlan, rp.noOutput, defaultFun = flashWedge)    
    exp.dispRunStats(expPlan)    


def plot(rp, plotData = None):
    print 'Meridian has no task, so there is nothing to plot'
    

def run(rp):
    runExp(rp)