###############################################################################
##                             Import modules                                ##
###############################################################################


# include system functions
import os


###############################################################################
##                                 Functions                                 ##
###############################################################################


def qMakedirs(path):
    """
    Recursively makes the path and completes silently if the path exists
    
    Makes os.makedirs friendlier
    """
    if not os.path.isdir(path):
        os.path.makedirs(path)
        

def listDir(path = '.', ending = '', pattern = '.*', cropExt = False, fullPath = True):
    """
    Get list of files of particular extensions
    
    Choose cropExt = True if you want the returned file list be without the
    extension.
    Choose fullPath = False if you want to get only file names with the path
    """
    import re
        
    if fullPath:
        addPath = os.path.normpath(path) + '/'
    else:
        addPath = ''
    
    fileList = [os.path.split(f)[1] for f in os.listdir(path)]
    output = []
    
    
    for f in fileList:
        takeIt = False
        if ending != '':
            if f.endswith(ending): takeIt = True
        elif re.search(pattern, f) != None: takeIt = True
        
        if takeIt:
            if cropExt:
                output.append(addPath + os.path.splitext(f)[0])
            else:
                output.append(addPath + f)
    return output  


# From Python 2.7 docs under the Python Software Foundation License
# http://docs.python.org/library/itertools.html#itertools.combinations
def combinations(iterable, r):
    # combinations('ABCD', 2) --> AB AC AD BC BD CD
    # combinations(range(4), 3) --> 012 013 023 123
    # from Python 2.6
    pool = tuple(iterable)
    n = len(pool)
    if r > n:
        return
    indices = range(r)
    yield tuple(pool[i] for i in indices)
    while True:
        for i in reversed(range(r)):
            if indices[i] != i + n - r:
                break
        else:
            return
        indices[i] += 1
        for j in range(i+1, r):
            indices[j] = indices[j-1] + 1
        yield tuple(pool[i] for i in indices)


# From Python 2.7 docs under the Python Software Foundation License 
# http://docs.python.org/library/itertools.html#itertools.combinations_with_replacement       
def combinations_with_replacement(iterable, r):
    # combinations_with_replacement('ABC', 2) --> AA AB AC BB BC CC
    pool = tuple(iterable)
    n = len(pool)
    if not n and r:
        return
    indices = [0] * r
    yield tuple(pool[i] for i in indices)
    while True:
        for i in reversed(range(r)):
            if indices[i] != n - 1:
                break
        else:
            return
        indices[i:] = [indices[i] + 1] * (r - i)
        yield tuple(pool[i] for i in indices)

        
def advCmp(a, b):
    """
    Compare two values with True, False, and None as a possible outputs
    """
    if b == None and a != None: return None
    else: return a == b

        
def frac(a):    
    """
    Returns the part after the dot of a given number
    """    
    return float('0.' + str(a).split('.')[1])
    

def makeUnique(path, fileName, ext = '.txt'):
    """
    Generate a unique file name, such that no accidental data overwrite happens
    """
    fileName = path + '/' + fileName
    tryFileName = fileName
    i = 2
    while os.path.exists(tryFileName + ext):
        tryFileName = fileName + '(' + str(i) + ')'
        i += 1
    return tryFileName + ext
    
    
def randDoubleList(list):
    """
    Generates a two number lists of equal length where
    """        
    items = [(random.random(), i) for i in list]        
    items.sort()
    items2 = [i for i in list]
    while items2 != []:
        out = []                        
        for item in items:
            couple = random.choice(items2)
            while couple == item[1] and items2!=[item[1]]:
                couple = random.choice(items2)
            items2.remove(couple)
            out.append([item[1], couple])
        if items2 != []:
            items2 = [i for i in list]
    return out

    
def listify(item, level = 0):
    """
    Checks if given variable is a depth %level% list, and if not, makes it such
    Depth %level% list means that 
    NOT CORRECT FOR ALL INPUTS YET!
    """
    if item == None: return []
    else:
        # this is a hack for now
        if level == 1:
            out = []
            for it in item:
                if not type(it) in [list, tuple]: out.append([it])
                else: out.append(list(it))
            return out
        #if level != 0:
        #    itemDepth = eval('item' + str([0]*level))
        else:
            itemDepth = item
        thisType = type(itemDepth).__name__
        if not thisType in ['list', 'tuple']:
            if thisType in ['array', 'ndarray']: 
                item = list(item)
            else:
                item = [item]
        return item
    
def getFileName(fullPathList):    
    fullPathList = listify(fullPathList)
    out = [pathName.split('/')[-1].split('.')[0] for pathName in fullPathList]
    return out
    
def genFileDict(fileList):
    fileDict = {}
    tempDict = {}
    #import pdb; pdb.set_trace()
    if isinstance(fileList, tuple): fileList = list(fileList)
    
    if isinstance(fileList, list):
        for thisFileList in fileList:
            out = listify(thisFileList)
            tempDict['-'.join(out)] = out
    elif not isinstance(fileList, dict): # just a single string
        tempDict[fileList] = [fileList]
    else: tempDict = fileList
    
    for key, value in tempDict.items():
        fileDict[key] = []
        for item in value:
            if item[0] <> '^': item = '.*' + item
            item += '.*'            
            fileDict[key].append(item)
        
def genFileList(fileList):
    
    try: fileList = eval(fileList)
    except: pass
    fileList = listify(fileList)
    if type(fileList) == tuple: fileList = list(fileList)
    outList = []
    
    
    for thisList in fileList:
        thisSubList = listify(thisList[0])
        temp = []
        
        for item in thisSubList:
            if item[0] <> '^': item = '.*' + item
            item += '.*'     
            temp.append(item)
            
        outList.append(temp)
        
    return outList
    

# OrderedDict code (because we're stuck with Python 2.5)
# Created by Raymond Hettinger on Wed, 18 Mar 2009, under the MIT License
# http://code.activestate.com/recipes/576693/
from UserDict import DictMixin

class OrderedDict(dict, DictMixin):

    def __init__(self, *args, **kwds):
        if len(args) > 1:
            raise TypeError('expected at most 1 arguments, got %d' % len(args))
        try:
            self.__end
        except AttributeError:
            self.clear()
        self.update(*args, **kwds)

    def clear(self):
        self.__end = end = []
        end += [None, end, end]         # sentinel node for doubly linked list
        self.__map = {}                 # key --> [key, prev, next]
        dict.clear(self)

    def __setitem__(self, key, value):
        if key not in self:
            end = self.__end
            curr = end[1]
            curr[2] = end[1] = self.__map[key] = [key, curr, end]
        dict.__setitem__(self, key, value)

    def __delitem__(self, key):
        dict.__delitem__(self, key)
        key, prev, next = self.__map.pop(key)
        prev[2] = next
        next[1] = prev

    def __iter__(self):
        end = self.__end
        curr = end[2]
        while curr is not end:
            yield curr[0]
            curr = curr[2]

    def __reversed__(self):
        end = self.__end
        curr = end[1]
        while curr is not end:
            yield curr[0]
            curr = curr[1]

    def popitem(self, last=True):
        if not self:
            raise KeyError('dictionary is empty')
        if last:
            key = reversed(self).next()
        else:
            key = iter(self).next()
        value = self.pop(key)
        return key, value

    def __reduce__(self):
        items = [[k, self[k]] for k in self]
        tmp = self.__map, self.__end
        del self.__map, self.__end
        inst_dict = vars(self).copy()
        self.__map, self.__end = tmp
        if inst_dict:
            return (self.__class__, (items,), inst_dict)
        return self.__class__, (items,)

    def keys(self):
        return list(self)

    setdefault = DictMixin.setdefault
    update = DictMixin.update
    pop = DictMixin.pop
    values = DictMixin.values
    items = DictMixin.items
    iterkeys = DictMixin.iterkeys
    itervalues = DictMixin.itervalues
    iteritems = DictMixin.iteritems

    def __repr__(self):
        if not self:
            return '%s()' % (self.__class__.__name__,)
        return '%s(%r)' % (self.__class__.__name__, self.items())

    def copy(self):
        return self.__class__(self)

    @classmethod
    def fromkeys(cls, iterable, value=None):
        d = cls()
        for key in iterable:
            d[key] = value
        return d

    def __eq__(self, other):
        if isinstance(other, OrderedDict):
            return len(self)==len(other) and self.items() == other.items()
        return dict.__eq__(self, other)

    def __ne__(self, other):
        return not self == other    
        
