"""
This module contains tools for renaming files
"""

import re, os
import qFunctions as q
import core.init


def renameFMRI(runParams):
    """
    Used to rename fMRI output files to func_SESSNO_RUNTYPE or anat_SESSNO
    """
    
    for subjID in runParams['subjID']:    
        dataFiles = os.listdir(init.paths['dataDir'] %subjID)
        for fileName in os.listdir(init.paths['datafmriDir'] %subjID):            
            if re.search('^' + subjID + ".*\.(PAR|REC|nii)", fileName):
                sessNo = '%02d' %int(fileName.split('_')[-2])
                outName = None
                # detect anatomical runs            
                if fileName.split('_')[-5] == '3DTFE':
                    #num = fileName.split('.')[0].split('_')[-1]
                    outName = 'anat_' + sessNo #+ '_' + num     
                else:
                    dataFileName = None
                    for dataFile in dataFiles:
                        if dataFile.split('_')[1] == sessNo:
                            dataFileName = dataFile
                            break
                    if dataFileName == None:
                        print 'Data file for session %s not found' %sessNo
                    else: outName = 'func_' + sessNo + '_' + dataFileName.split('.')[0].split('_')[-1]
                        
                if outName != None:
                    newFileName = outName + '.' + fileName.split('.')[1]
                    print fileName + ' -> ' + newFileName
                    if not runParams['noOutput']:
                        os.rename(init.paths['datafmriDir'] %subjID + fileName,
                            init.paths['datafmriDir'] %subjID + newFileName)


def renameFMRItemp(runParams):
    """
    This was only used once. Kept for record. More versatile that renameFMRI()
    """
    global paths
    
    dataFiles = q.listDir(paths['dataDir'], ending = 'csv', fullPath = False)
    for fileName in os.listdir(paths['datafmriDir']):            
        if re.search(runParams['subjID'], fileName):
            st = str.find(fileName,runParams['subjID'])
            pref = fileName[:st]
            st = str.find(fileName,'SENSE') + len('SENSE')
            sessNo = fileName[st+1:st+3]
            post = fileName[st+5:]
            
            outName = None
            # detect anatomical runs            
            if str.find(fileName,'3DTFE') != -1: # success in finding '3DTFE' in the file name
                if str.find(fileName,'MPR') == -1: # don't take 'MPR' anatomicals
                    
                    sessNo = fileName[st+1:st+2]
                    post = fileName[st+4:]
                    outName = pref + 'anat_0' + sessNo + post
                else: os.remove(paths['datafmriDir']+fileName)
            else:
                dataFileName = None
                for dataFile in dataFiles:
                    if dataFile.split('_')[1] == sessNo:
                        dataFileName = dataFile
                        break
                if dataFileName == None:
                    print 'Data file for session %s not found' %sessNo
                else: outName = pref + 'func_' + sessNo + '_' + dataFileName.split('.')[0].split('_')[-1] + post
                    
            if outName != None:
                #newFileName = outName + '.' + fileName.split('.')[1]
                print fileName + ' -> ' + outName
                os.rename(paths['datafmriDir']+fileName, paths['datafmriDir']+outName)


if __name__ == "__main__":
    renameFMRI(options)
